
# README: Nucleoid Analysis Code for Kleckner group

This codebase analyzes nucleoid data for the Kleckner group.

## Algorithm/Approach

1. Read in 3D data and scale based on stack statistics.

2. Use thresholding and 3D watershed functionality to identify the shapes vs. noise. Essentially, the code segments the scaled volume into separate objects and returns two stacks, one that contains the segmented objects and a second that labels each voxel with an object's resulting ID number assigned by the segmentation code.

3. Create a path for the core of each identified shape by, computing  the intensity-weighted centroid position, iterating one slice at a time through the scanaxis.

4. Convert the path data to cylindrical (polar) coordinates in order to define the radius and angle (theta) for each axis position.

5. Calculate pitch as the inverted, first-derivative of the smoothed angle for each axis position.

6. Changes in handedness occur where the theta-derivative crosses zero. Postive changes in angle depict one direction of movement (clockwise or counter-clockwise) while negative changes in angle match movement in the opposite direction. 

## Code Breakdown

Code for this analysis comes in four general types:

* Data input: loading, scaling, and/or cleaning
* Analysis: volume segmentation, centroid localization, coordinate transforms, derived computations for output
* Testing/Modeling: scripts and functions written for testing the functionality of the algortihms and the effectiveness of the visualization functions.
* Visualization: Graphically plotting the results as volumes or 2D derived values. 

## Data Input
  * `tiff2matrix`: loads a tiff file into a 3D matrix 'stack'.
  * `scale_stack`: scales a stack according to the stack's maximum value and outputs the correct quantities in the designated datatype, i.e. `single`, `double`, `uint8`, etc...

## Computation/Analysis
The `nucleoid_centroid` function is used to do **general analysis** of a given TIF volume. Its parameters include:

  * _filename_: the _filename_ for the TIF volume to be analyzed.
  * _xyzscales_: A 3-element vector containing the scale conversions for X, Y, and Z coordinates in pixels/micron (default: [1 1 1]).
  * _sfactor_:  The smoothing factor when computing pitch. It defaults to 0.5, which is roughly what a simple/naive forward-difference derivative would return. Think of it as the standard deviation between collected points. A value of 2 would essentially smooth a point with its nearest neighboring points on both sides.
  * _normvolreject_: Rejection volume fraction (default 0.15). The analysis will reject any 3D objects it finds to have less than _normvolreject_ of the total volume. A _normvolreject_ of 0.15 will reject objects whose volumes occupy less than 15% of the total.
  * _watershed_threshold_:  manually defines a threshold for the watershed rejection minimum, whose default is 0.5 (50% of normalized-stack max). Example: a value of 0.75 would assign "valleys" between objects whose values are less than 75% of the stack maximum as places to segment between two objects.
  * _scanaxis_: Axis of nucleoid alignment, i.e. `'X'`, `'Y'`, or `'Z'` (default: `'Y'`). More specifically, it is the axis along which the nucleoid is aligned while in the "lab-frame". The "lab-frame" is defined as the coordinate system with its origin in the center of a microscope's illuminated field-of-view, where:
     * +X points to the right
     * +Y points away from the microscopist
     * -Z points "down", in the direction of Earth's gravity

  The output of `nucleoid_centroid` is a MATLAB data structure that contains the inputted, scaled, and cleaned volumes along with all of the analyzed results, i.e. centroids, pitch, handedness, etc... **outputted in the lab-frame**.

  A typical use-case: 

    LF = nucleoid_centroid('narrow_LF_3Dstack_A1_ROI4.tif', [1 1 1], 0.5, 0.07, 0.5, 'X');

  Here, `LF` is the outputted nucleoid data structure that contains all of the inputs and analysis. Typing `LF` into the MATLAB command window and pressing enter returns a listing of the output that should look similar to this:

    struct with fields:

         metadata: [1×1 struct]
         scanaxis: 'X'
            stack: [1×1 struct]
        centroids: [1×1 struct]
              rph: [1×1 struct]
## Outputting/Exporting Dataset

  The data can be outputted as "pseudoatoms" for import into PyMol using `save_for_pymol`.

   Usage:

     save_for_pymol(nucleoid, myfilename)

   Where `nucleoid` is the variable outputted by `nucleoid_centroid` and `my_filename` is a string containing the exact filename the data should be outputted to. This function is used in the `test_nucleoids` script, so a good usage example can be found there.

## Visualizing the output

Surface, or 3-D, visualizations in this package are designed as "ready-to-rotate". Click in the volume and drag to change the visualization angle.

  * To see the segmented objects in the volume, use `plot_isovolume`, which requires three inputs:
     * _stack_: imaging data as a volume, arranged as an HxWxN matrix, with N images, each at HxW resolution
     * _isovalue_ (optional): range 0-255, the intensity value to use for isovalue. Defaults to the mean intensity of the volume.
     * _scales_: A 3-element vector containing the scale conversions for X, Y, and Z coordinates in pixels/micron (default: [1 1 1]).
     * _ghost_ (optional): (`true`, `false`), defaults to `false`. When `true`, instead of plotting the isosurface in an opaque green, use a semi-transparent white instead. Useful for visualizing the centroids aligned within the volumes of interest.

   A typical use case: `plot_isovolume(LF.stack.scaled, [], scales, false);`

   This particular use case plots an opaque-green isovolume in a rotatable volume. Since the second argument is empty, i.e. the '`[]`', it uses the default `isovalue`, i.e. the scaled-stack mean).


  * To see the centroids plotted within their nucleoid volumes, use `plot_centroids3d`, which has one input, the `nucleoid` structure outputted from the `nucleoid_centroid` function. Visualizing the example used above, try:

        plot_centroids3d(LF);

  
## Demonstration/Testing



---

## Notes

Some files were imported from earlier code.

**Imported Files**

* `aquisition_picked`: Drives the microscope. Where the experimental information is selected and the header is created.
* `createTiffGW.m`, `fillTiffGW.m`, `metadataGW.m`: Make tiffs and store their info.
* `General_caller`: data analysis script that calls a number of functions including TiffLoad_partial_multi.

---

