function centroid = get_centroids(segmented_stack, objid_stack, xyzscales, scanaxis)
% GET_CENTROIDS computes the standard and intensity-weighted centroids
%
% outs = get_centroids(segmented_stack, objid_stack, scanaxis)
%
% Computes the centroid positions for a series of segmented objects along 
% the "scanaxis", located in the "segmented_stack" and identified by the 
% "objid_stack".
%
% Inputs:
% - segmented_stack (MxNxP-sized 3D matrix) containing stack of segmented 
%   fluorescence images, outputted by segment_volume function.
% - objid_stack (must be same size as segmented_stack) also outputted by
%   the "segment_volume" function.
%
% Output:
% - "outs" is a structure that contains the list of objectIDs and the XYZ
%   locations for the computed centroids, both the basic-average centroid 
%   and the intensity-weighted centroid.
%

if nargin < 4 || isempty(scanaxis)
    scanaxis = 'y';
end

if nargin < 3 || isempty(xyzscales)
    xyzscales = [1 1 1];
end

if nargin < 2 || isempty(segmented_stack) || isempty(objid_stack)
    error('Both segmented stack and Object ID stacks are required.')
end

segmented_stack = double(segmented_stack);
objid_stack = double(objid_stack);

objids = unique(objid_stack);

scanaxis = upper(scanaxis);

[segmented_stack, rotxyzscales] = rot_scanaxis(segmented_stack, xyzscales, scanaxis, 'scan');
objid_stack = rot_scanaxis(objid_stack, xyzscales, scanaxis, 'scan');

K = size(segmented_stack,3);
M = length(objids);

count = 1;

% For each object...
for m = 1 : M
    
    thisobj = objids(m);

    % ignore the background (object 0)
    if ~thisobj
        continue;
    end
    
    objmask = zeros(size(objid_stack));
    objmask(objid_stack == thisobj) = objid_stack( objid_stack == thisobj );    
    objmask = logical(objmask);
    
    % Iterate through each frame in the stack...
    for k = 1:K
        thisimage = squeeze(segmented_stack(:,:,k)); 
        thisobjmask = squeeze(objmask(:,:,k));

        intensimage = thisimage .* thisobjmask;
        intensimage = intensimage ./ max(intensimage(:));
          
        [imy,imx] = find(thisobjmask);    

        
          if isempty(imy)
              continue;
          else
              ObjID(count,1) = thisobj;

              [cx, cy, sx, sy] = centerofmass(intensimage);
              XYZ(count,1) = cx; % PERMANENT SCALING HAPPENS HERE
              XYZ(count,2) = cy;
              XYZ(count,3) = k;
                            
          end
      
          count = count + 1;
    end
    
end

% Need to get XYZ coordinates appropriately scaled BEFORE converting into
% cylindrical coordinate system.
ScaledXYZ = XYZ .* repmat(xyzscales,size(XYZ,1),1);

% convert to cylindrical coordinates with origin at Zcyl = 0
TRZ = centroid_cylcoords(ObjID, ScaledXYZ);

% Need to get the output coordinates back in the lab frame. These will
% depend on the plane used for the centroids
%
switch scanaxis
    case 'X'
        XYZ(:,[1 2 3]) = XYZ(:,[3 2 1]);
    case 'Y'
        XYZ(:,[1 2 3]) = XYZ(:,[1 3 2]);
    case 'Z'
        % nothing needs to change
    otherwise
        % given earlier switch, this should never happen

end
        
    centroid.ObjID     = ObjID;
    centroid.X = XYZ(:,1);
    centroid.Y = XYZ(:,2);
    centroid.Z = XYZ(:,3);      % X   Y   Z  (lab-frame) in SCALED DISTANCE
    centroid.Theta = TRZ(:,1);
    centroid.Radius = TRZ(:,2);
    centroid.Zcyl = TRZ(:,3);      % neutralized, scaled, z-aligned, cylindrical coordinates
    
return;


function centroid_cyl = centroid_cylcoords(objid, centroidxyz) %, scanaxis)

    % BUT BEFORE doing that, while the centroid is aligned with the Z axis,
    % let's make a version converted to cylidrical coords.
    % first need to subtract off offsets
    
    objlist = unique(objid);
    
    nObj = length(objlist);
    
    centroidtrz = centroidxyz;
    
    for k = 1:nObj

        thisobj = objid == objlist(k);
        thisnuclxyz = centroidtrz( thisobj , :);
        
        x = thisnuclxyz(:,1) - mean(thisnuclxyz(:,1));
        y = thisnuclxyz(:,2) - mean(thisnuclxyz(:,2));
        z = thisnuclxyz(:,3) - min(thisnuclxyz(:,3));
        

        [theta, rho, cylz] = cart2pol(x, y, z); 
        
        theta = unwrap(theta);
        
        centroidtrz(thisobj, :) = [theta, rho, cylz];        

    end  

    centroid_cyl = centroidtrz;
    
return;
