function [outs, tplocs] = get_rph(centroids, sfactor)
% GET_RPH collects & calculates radius, pitch, and handedness functions for nucleoid datasets
%
% h = get_rph(centroids, sfactor)
%
% Inputs:
%   centroids = centroid table outputted by get_centroids
%   sfactor = smoothing factor, defaults to 0.5, which is roughly what a 
%           simple forward difference would return. Think of it as the
%           standard deviation between collected points. A value of 2 would
%           basically smooth a point with its nearest two points on both
%           sides.
% Outputs:
%   h = handle to plotted figure
%

    % The objects identified by the segmentation code are housed in a 2D list  owned by the
    % centroids datastructure. Consider promoting this to a matlab table object.
    objid = centroids.ObjID;
    
    % Get the count/number and list of object IDs we have to work with
    objlist = unique(objid);    
    nObj = length(objlist);    
    
    count = 1;
    
    % Across the list for each object...
    for k = 1:nObj

        % Extract the kth object's cylindrical coordinates, where T is the 
        % angle, theta, in radians, R is the cylinder's radius, and Z is 
        % the current height/posiion of the cylinder.
        thisobj = (objid == objlist(k));
        
        theta = centroids.Theta( thisobj, :);
        rho   = centroids.Radius( thisobj, :);
        zcyl     = centroids.Zcyl( thisobj, :);
        
        % The pitch is related to the change in sign of the angle theta's derivative.
        % First, calculate the derivative. The CreateGaussScaleSpace function comes 
        % UNC's CISMM group. It includes a smoothing factor, here labeled as 'sfactor'
        % and is also vector-size friendly in that the returned vector contains  no 
        % missing points like those seen in simple first-forward-difference computations.
        dz = zcyl(2) - zcyl(1); 
        dtheta_dzcyl = CreateGaussScaleSpace(theta, 1, sfactor) / dz;
        
        % Pitch calculation
        mypitch = (2*pi) ./ abs(dtheta_dzcyl);
                
        % Locates tendril perversions as Zero-Crossing Indices Of Argument Vector
        zci = @(v) find(v(:).*circshift(v(:), [-1 0]) <= 0);                    
        locs = zci(dtheta_dzcyl);
        tendril_perv_locs = locs(1:end-1); % "end-1" because I think the last one is an artifact
        
        centroids.Pitch(thisobj,1) = mypitch;
        centroids.Handed(thisobj,1) = dtheta_dzcyl;
                
        tplocs{k,1} = tendril_perv_locs;
        
    end
    
    outs = struct2table(centroids);

    % Add metadata to the table.    
    outs.Properties.Description = 'Nucleoid Centroid Data';
    outs.Properties.VariableUnits{'X'} = 'pixels';
    outs.Properties.VariableUnits{'Y'} = 'pixels';
    outs.Properties.VariableUnits{'Z'} = 'pixels';
    outs.Properties.VariableUnits{'Theta'} = 'radians';
    outs.Properties.VariableUnits{'Radius'} = 'um';
    outs.Properties.VariableUnits{'Zcyl'} = 'um';
    outs.Properties.VariableUnits{'Pitch'} = 'um';
    outs.Properties.VariableUnits{'Handed'} = 'radians/um';
    outs.Properties.VariableDescriptions{'Theta'} = 'Unwrapped phase angle';
    
    return
