function nucl = nucleoid_centroid(tiff_file, xyzscales, sfactor, normvolreject, watershed_threshold, scanaxis)
% NUCLEOID_CENTROID performs basic anaysis on nucleoid volumes
%
% nucl = nucleoid_centroid(tiff_file, normvolreject, scanaxis)
%
% Given a tiff_file, nucleoid_centroid performs a basic analysis routine on
% a tiff stack containing a labelled cellular nucleoid aligned in the 
% 'scanaxis' direction. The inputted stack is scaled to the stack's maximum 
% value and then segmented into likely objects based on a watershed 
% algorithm. Objects containing a volume fraction less than 'normvolreject' 
% are ignored.
%
% Inputs:
%    
%    tiff_file: 3D tiff stack filename, no default
%    xyzscales: scales to convert pixels to microns, default [1 1 1]
%               microns/pixel.
%    sfactor: smoothing factor when computing pitch. Default: 0.5
%    normvolreject: rejection volume fraction, default 0.15
%    watershed_threshold: fraction threshold for watershed segmentation,
%                         default set at 0.5.
%    scanaxis: 'X' | {'Y'} | 'Z'
%
% Output: 
%
%    nucleoid structure containing original stack, scaled stack with
%    scales, segmented and object ID stacks, centroid locations, radius,
%    handedness, and pitch measurements
%

if nargin < 6 || isempty(scanaxis)
    scanaxis = 'Y';
end

if nargin < 5 || isempty(watershed_threshold)
    watershed_threshold = 0.5;
end

if nargin < 4 || isempty(normvolreject)
    normvolreject = 0.15;
end

if nargin < 3 || isempty(sfactor)
    sfactor = 0.5;
end

if nargin < 2 || isempty(xyzscales)
    xyzscales = [1 1 1];
end

if nargin < 1 || isempty(tiff_file)
    error('No tiff stack to work with.');
end

scanaxis = upper(scanaxis);

% put in testing for scanaxis to throw error if not x, y, or z

% Handle input as either a 3D matrix containing stack data or load in a tiff
% file and handle it that way
if isnumeric(tiff_file)
    tiffstack = tiff_file;
    tiffmeta = 0;
else
    [tiffstack tiffmeta] = tiff2matrix(tiff_file);
end

% Scale the entire stack according to stack's maximum and minimum as
% opposed to each image's stats.
[scaledstack, scales]        = scale_stack(tiffstack);

% Segment the scaled volume into separate objects and return both the volume/stack
% containing the segmented objects as well as a volume containing each objects
% resulting ID number given to it by the segmentation code.
[segstack, objects_in_stack] = segment_volume(scaledstack, normvolreject, watershed_threshold);

% Compute the intensity-weighted centroid locations as the code steps through 
% the 'scanaxis'. Output cents structure contains resuls in both Cartesian and
% cylindrical coordinate systems for convenience
cents = get_centroids(segstack, objects_in_stack, xyzscales, scanaxis);

% Use cylindrical coordinates and a smoothing factor to compute the 
% radius, pitch, and handedness of each nucleoid's centroid locations
[cents, tplocs] = get_rph(cents, sfactor);

sec = section_stats(cents, tplocs);

Nucl.Filename = tiff_file;
Nucl.ScalesXYZ = xyzscales;
Nucl.ScanAxis = scanaxis;
Nucl.SmoothFactor = sfactor;
Nucl.VolFracReject = normvolreject;
Nucl.WaterThresh = watershed_threshold;
Nucl.OrigStack = tiffstack;
Nucl.SegStack = segstack;
Nucl.ObjIdStack = objects_in_stack;
Nucl.Cents = cents;
Nucl.Sec = sec;
Nucl.TpLocs = tplocs;

nucl = Nucl;

return
