function [h,isovalue] = plot_isovolume(stack, isovalue, xyzscales, ghostTF)
% PLOT_ISOVOLUME plots a simple microscopy volume as an isosurface 
%
% h = plot_isovolume(stack, isovalue, ghostTF)
%
% Inputs:
%    stack: HxWxN matrix, N images with HxW resolution
%    isovalue (optional): 0-255, the intensity value to use for isovalue
%    xyzscales (optional): [1 1 1], microns per pixel value for x, y, and z
%    ghostTF (optional): true | {false}, plots semi-transparent isosurface
%
% Output:
%    h: handle to plotted figure

    if nargin < 1 || isempty(stack)
        error('No stack defined.');
    end

    stack = scale_stack(stack, 'uint8');

    if nargin < 2 || isempty(isovalue)
        mm = mean(double(stack(:))) + std(double(stack(:)));
        isovalue = uint8(floor(mm));
    end
    
    if nargin < 3 || isempty(xyzscales)
        xyzscales = [1 1 1];
    end
    
    if nargin < 4 || isempty(ghostTF)
        ghostTF = false;
    end
    
    [m,n,p] = size(stack);
    [X,Y,Z] = meshgrid(1:n,1:m,1:p);
    [xs, ys, zs] = deal(xyzscales(1), xyzscales(2), xyzscales(3));
    
    Jiso = isosurface(X.*xs, Y.*ys, Z.*zs, stack,isovalue);

    h = figure; 
    
    p = patch(Jiso);
    
    if ghostTF
        set(p, 'EdgeColor', [0.65 0.65 0.65]);
        set(p, 'EdgeAlpha', 0.3);
        set(p, 'FaceColor', [1 1 1]);
        set(p, 'FaceAlpha', 0.3);
    else
        set(p, 'FaceColor', [0, 0.5, 0]);
        set(p, 'EdgeAlpha', 0.3);
    end
    grid on;

    xlabel('X');
    ylabel('Y');
    zlabel('Z');

%     axis square;
    axis equal;
    camproj('orthographic');
    
    set(gca, 'Units', 'normalized');
    set(gca, 'Position', [0.2 0.2 0.6 0.6]);

    cameratoolbar('SetMode', 'orbit');
    
    
return;
