function [outs, rotxyzscales] = rot_scanaxis(stack, xyzscales, scanaxis,  coord)
% ROT_SCANAXIS rotates lab-frame to scan-axis coordinates for nucleoid stacks
%
%   outs = rot_scanaxis(stack, xyzscales, scanaxis, coord)
%
% stack = MxNxP matrix containing nucleoid imaging data
% xyzscales = lab-frame [X Y Z] scaling factors in [um/px], default [1 1 1]
% scanaxis = 'X' | {'Y'} | 'Z'
% coord = {'lab'} | 'scan' denotes the coordinate-orientation of the
% desired output, ASSUMING the input is alread in the alternate orientation. 
%

    % Lab-frame coordinates: As user sits in front of microscope, -Z
    % is in the direction of gravity, +Y is away from and -X to the left
    % of the microscopist.
    if nargin < 4 || isempty(coord)
        coord = 'lab';
    end
    
    % Because most (if not all) of the data I've seen come out of the 
    % Kleckner group shows the nucleoids aligned in the Y direction,
    % I felt comfortable putting that as a default input.
    if nargin < 3 || isempty(scanaxis)
        scanaxis = 'y';
    end
    
    if nargin < 2 || isempty(xyzscales)
        xyzscales = [1 1 1];
    end
    
    if nargin < 1 || isempty(stack)
        error('An image stack is required.')
    end

    scanaxis = upper(scanaxis);

    % The algortihm rotates the inputted volume such that the 
    % nucleoid is aligned with lab-frame Z. This makes addressing
    % and assignment in the code easier to follow. This rotation
    % is handled by matlab's "permute" function and depends on the
    % scanaxis defined by the user on the input.
    switch scanaxis
        case 'Y'
            P = [3 2 1];
        case 'X'
            P = [1 3 2];
        case 'Z'
            P = [1 2 3];
        otherwise
            error('Scanaxis value must be "X", "Y", or "Z"');
    end

    % Rotation back into the scan frame uses the "ipermute" matlab 
    % function which uses the same permute-axis vector assignment
    switch coord
        case 'scan'
            outs = permute(stack, P);
            rotxyzscales = xyzscales(P);
        case 'lab'
            outs = ipermute(stack, P);
            rotxyzscales = xyzscales(P);
        otherwise
            error('coord must denote the "scan" or "lab" frame.');
    end

return
