function [segstack, objStack] = segment_volume(stack, rejectVnotbkgdVolume, watershed_threshold, debugplotTF)
% SEGMENT_VOLUME segments a volume dataset, optimized for nucleoid data
%
% [segstack, objStack] = segment_volume(stack, rejectVnotbkgdVolume, watershed_threshold)
%
% Segments a nucleoid data-volume into separate objects using watershed
% segmentation. Rejects objects with volumes smaller than
% "rejectVnotbkgdVolume" and rejects segments with values less than a 
% "watershed_threshold".
%
% Inputs:
% - "stack" (MxNxP-sized 3D matrix) containing stack of fluorescence images 
%   outputted by tiff2matrix or scale_stack functions.
% - "rejectVnotbkgdVolume" normalized rejection volume. A value of "0.5" 
%   would reject objects containing < 50% of the full imaging volume. A 
%   typically "good" value to start with here in nucloid volumes is 0.07.
% - "watershed_threshold" manually defines a threshold for the watershed
%   rejection minimum, whose default is 0.5 (50% of normalized-stack max)
%
% Output:
% - "segstack", a MxNxP matrix containing the original, but segmented, stack
% - "objStack", a MxNxP matrix labeling voxels with ObjectIDs
%

if nargin < 4 || isempty(debugplotTF)
    debugplotTF = false;
end

if nargin < 3 || isempty(watershed_threshold)
    watershed_threshold = 0.5;
end


if nargin < 2 || isempty(rejectVnotbkgdVolume)
    rejectVnotbkgdVolume = 0.05;
end

% create a threshold for the stack
J = imbinarize(stack);
if debugplotTF
    plot_slice_volume(J);
    drawnow;
end

% eliminate anything outside that which exceeds the image/binarized threshold
Jst = zeros(size(stack));
Jst(J) = stack(J);
if debugplotTF
    plot_slice_volume(Jst);
    drawnow;
end
    
% compute the distance function from the complement of the stack
D = bwdist(imcomplement(J));
if debugplotTF
    plot_slice_volume(D);
    drawnow;
end

% invert and set voxels not part of the objects to Inf
D = -D;
D(~J) = Inf;
if debugplotTF
    plot_slice_volume(D);
    drawnow;
end

% H = imhmin(D, 0.5);
H = imhmin(D, watershed_threshold);
if debugplotTF
    plot_slice_volume(H);
    drawnow;
end

% compute the watershed
L = watershed(H);
L(~J) = 0;
if debugplotTF
    plot_slice_volume(L);
    drawnow;
end

% Objects in volume
oiv = unique(L);
Nobjects = length(oiv);
Vcat = categorical(L(:));
[Vobj ObjName] = histcounts(Vcat);
Vobj = Vobj(:);
ObjName = cellfun(@str2num, ObjName, 'UniformOutput', false);
ObjName = cell2mat(ObjName(:));
Vnormalized = Vobj/length(L(:));
Vnotbkgd = Vobj/length(L(L>0));
VolTable = table(ObjName, Vobj, Vnormalized, Vnotbkgd, 'VariableNames', {'ObjectName', 'Vobj', 'Vnormalized', 'Vnotbkgd'});

% Reject volumes smaller than some threshold percentage
Obj_reject = ObjName(Vnotbkgd < rejectVnotbkgdVolume);
for k = 1:length(Obj_reject)
    L(L == Obj_reject(k)) = 0;
end

% objStack = uint16(L);
objStack = double(L);

% output the original volume with segments cut out
segstack = zeros(size(stack));
segstack(logical(L)) = stack(logical(L));
segstack = double(segstack);
% outstack = uint16(outstack);

if debugplotTF
    plot_slice_volume(segstack);
    drawnow;
end

return
